#!/usr/bin/env python
# -*- coding: utf 8 -*-
"""
Unit conversion.

:copyright: 2015 Agile Geoscience
:license: Apache 2.0
"""


def ft_m(x):
    return x / 0.3048


def m_ft(x):
    return x * 0.3048
